<?php
/*
 * Tutorial: How to Disable Attachment Pages in WordPress
 * URL: https://www.rafaelcardero.com/tutorials/how-to-disable-attachment-pages-in-wordpress/
 */

/* -----------------------------------------------------------------------------
 * - Steps
 * ---------------------------------------------------------------------------*/

/*
 * Performs a redirect from an attachment page to the underlying file.
 */
function ns_redirect_attachment(){

    if(is_attachment()){
        $attachment_ID = get_queried_object_id();
        $file_url = wp_get_attachment_url($attachment_ID);
        wp_redirect($file_url, 301);
        exit();
    }
}
add_action('template_redirect', 'ns_redirect_attachment', 10, 0);

/*
 * Replaces the attachment page URL with the URL of the underlying file.
 */
function ns_replace_attachment_link($link, $post_ID){
	return wp_get_attachment_url($post_ID) ?: $link;
}
add_filter('attachment_link', 'ns_replace_attachment_link', 10, 2);

/* -----------------------------------------------------------------------------
 * - Exercise
 * ---------------------------------------------------------------------------*/

/*
 * Performs a redirect from an attachment page to the parent post, if any, or to
 * the underlying file.
 */
function ns_redirect_attachment_conditionally(){
    
    global $post;
    
    if(is_attachment()){
        $redirect_url = $post->post_parent ? get_permalink($post->post_parent) : wp_get_attachment_url($post->ID);
        wp_redirect($redirect_url, 301);
        exit();
    }
}
add_action('template_redirect', 'ns_redirect_attachment_conditionally', 10, 0);

/*
 * Replaces the attachment page URL with the URL of the parent post, if any, or
 * with the URL of the underlying file.
 */
function ns_replace_attachment_link_conditionally($link, $post_ID){
    
    $post = get_post($post_ID);
    if($post){
        $link = $post->post_parent ? get_permalink($post->post_parent) : wp_get_attachment_url($post->ID);
    }
    return $link;
}
add_filter('attachment_link', 'ns_replace_attachment_link_conditionally', 10, 2);